@@ -9,6 +9,7 @@ of in shell scripts, batch files, or Makefiles.
 """
 
 import contextlib
+import datetime
 import fnmatch
 import glob
 import inspect
@@ -55,7 +56,7 @@ def do_show_env():
         print(f"  {env} = {os.environ[env]!r}")
 
 
-def do_remove_extension():
+def do_remove_extension(*args):
     """Remove the compiled C extension, no matter what its name."""
 
     so_patterns = """
@@ -65,8 +66,22 @@ def do_remove_extension():
         tracer.*.pyd
         """.split()
 
+    if "--from-install" in args:
+        # Get the install location using a subprocess to avoid
+        # locking the file we are about to delete
+        root = os.path.dirname(subprocess.check_output([
+            sys.executable,
+            "-Xutf8",
+            "-c",
+            "import coverage; print(coverage.__file__)"
+        ], encoding="utf-8").strip())
+    else:
+        root = "coverage"
+
     for pattern in so_patterns:
-        pattern = os.path.join("coverage", pattern)
+        pattern = os.path.join(root, pattern.strip())
+        if VERBOSITY:
+            print(f"Searching for {pattern}")
         for filename in glob.glob(pattern):
             if os.path.exists(filename):
                 if VERBOSITY:
@@ -365,6 +380,58 @@ def do_quietly(command):
     return proc.returncode
 
 
+def do_cheats():
+    """Show a cheatsheet of useful things during releasing."""
+    import coverage
+    ver = coverage.__version__
+    vi = coverage.version_info
+    shortver = f"{vi[0]}.{vi[1]}.{vi[2]}"
+    anchor = shortver.replace(".", "-")
+    if vi[3] != "final":
+        anchor += f"{vi[3][0]}{vi[4]}"
+    now = datetime.datetime.now()
+    branch = subprocess.getoutput("git rev-parse --abbrev-ref @")
+    print(f"Coverage version is {ver}")
+
+    print(f"pip install git+https://github.com/nedbat/coveragepy@{branch}")
+    print(f"https://coverage.readthedocs.io/en/{ver}/changes.html#changes-{anchor}")
+
+    print("\n## for CHANGES.rst before release:")
+    print(f".. _changes_{anchor}:")
+    print()
+    head = f"Version {ver} — {now:%Y-%m-%d}"
+    print(head)
+    print("-" * len(head))
+
+    print("\n## For doc/conf.py before release:")
+    print("\n".join([
+        '# The short X.Y.Z version.                                 # CHANGEME',
+        f'version = "{shortver}"',
+        '# The full version, including alpha/beta/rc tags.          # CHANGEME',
+        f'release = "{ver}"',
+        '# The date of release, in "monthname day, year" format.    # CHANGEME',
+        f'release_date = "{now:%B %-d, %Y}"',
+    ]))
+
+    print(
+        "\n## For GitHub commenting:\n" +
+        "This is now released as part of " +
+        f"[coverage {ver}](https://pypi.org/project/coverage/{ver})."
+    )
+    print("\n## For version.py next:")
+    next_vi = (vi[0], vi[1], vi[2]+1, "alpha", 0)
+    print(f"version_info = {next_vi}".replace("'", '"'))
+    print("\n## For CHANGES.rst after release:")
+    print(textwrap.dedent("""\
+        Unreleased
+        ----------
+
+        Nothing yet.
+
+
+        """))
+
+
 def do_help():
     """List the available commands"""
     items = list(globals().items())
