@@ -5,16 +5,19 @@
 
 import collections
 import contextlib
-import glob
 import os
 import os.path
 import re
 import shutil
 import subprocess
 import textwrap
+import warnings
 
 from unittest import mock
 
+import pytest
+
+from coverage import env
 from coverage.exceptions import CoverageWarning
 from coverage.misc import output_encoding
 
@@ -44,7 +47,7 @@ def run_command(cmd):
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT
-        )
+    )
     output, _ = proc.communicate()
     status = proc.returncode
 
@@ -87,6 +90,13 @@ def make_file(filename, text="", bytes=b"", newline=None):
     with open(filename, 'wb') as f:
         f.write(data)
 
+    # For debugging, enable this to show the contents of files created.
+    if 0:       # pragma: debugging
+        print(f"   ───┬──┤ {filename} ├───────────────────────")
+        for lineno, line in enumerate(data.splitlines(), start=1):
+            print(f"{lineno:6}│ {line.rstrip().decode()}")
+        print()
+
     return filename
 
 
@@ -127,7 +137,7 @@ class CheckUniqueFilenames:
         """The replacement method.  Check that we don't have dupes."""
         assert filename not in self.filenames, (
             f"File name {filename!r} passed to {self.wrapped!r} twice"
-            )
+        )
         self.filenames.add(filename)
         ret = self.wrapped(filename, *args, **kwargs)
         return ret
@@ -162,12 +172,6 @@ def re_line(pat, text):
     return lines[0]
 
 
-def remove_files(*patterns):
-    """Remove all files that match any of the patterns."""
-    for pattern in patterns:
-        for fname in glob.glob(pattern):
-            os.remove(fname)
-
 def remove_tree(dirname):
     """Remove a directory tree.
 
@@ -301,3 +305,20 @@ def assert_coverage_warnings(warns, *msgs):
             assert expected.search(actual), f"{actual!r} didn't match {expected!r}"
         else:
             assert expected == actual
+
+
+@contextlib.contextmanager
+def swallow_warnings(message=r".", category=CoverageWarning):
+    """Swallow particular warnings.
+
+    It's OK if they happen, or if they don't happen. Just ignore them.
+    """
+    with warnings.catch_warnings():
+        warnings.filterwarnings("ignore", category=category, message=message)
+        yield
+
+
+xfail_pypy_3749 = pytest.mark.xfail(
+    env.PYVERSION[:2] == (3, 8) and env.PYPY and env.PYPYVERSION >= (7, 3, 10),
+    reason="Avoid a PyPy bug: https://foss.heptapod.net/pypy/pypy/-/issues/3749",
+)
