@@ -1,7 +1,8 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt
+# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
 import pkg1.p1a, pkg1.p1b, pkg1.sub
 import pkg2.p2a, pkg2.p2b
 import othermods.othera, othermods.otherb
 import othermods.sub.osa, othermods.sub.osb
+import ambiguous, ambiguous.pkg1.ambiguous
