@@ -26,6 +26,7 @@ from tests.coveragetest import CoverageTest, TESTS_DIR, UsingModulesMixin
 from tests.helpers import assert_count_equal, assert_coverage_warnings
 from tests.helpers import change_dir, nice_file, os_sep
 
+BAD_SQLITE_REGEX = r"file( is encrypted or)? is not a database"
 
 class ApiTest(CoverageTest):
     """Api-oriented tests for coverage.py."""
@@ -422,7 +423,7 @@ class ApiTest(CoverageTest):
         self.make_file(".coverage.foo", """La la la, this isn't coverage data!""")
         cov = coverage.Coverage()
         warning_regex = (
-            r"Couldn't use data file '.*\.coverage\.foo': file is not a database"
+            r"Couldn't use data file '.*\.coverage\.foo': " + BAD_SQLITE_REGEX
         )
         with self.assert_warnings(cov, [warning_regex]):
             cov.combine()
@@ -539,7 +540,7 @@ class ApiTest(CoverageTest):
             "Module xyzzy was never imported. (module-not-imported)",
             "Module quux was never imported. (module-not-imported)",
             "No data was collected. (no-data-collected)",
-            )
+        )
 
     def test_warnings_suppressed(self):
         self.make_file("hello.py", """\
@@ -865,7 +866,11 @@ class SourceIncludeOmitTest(IncludeOmitTestsMixin, CoverageTest):
         # Since we need to import from there, we also add it to the beginning
         # of sys.path.
 
-        shutil.copytree(nice_file(TESTS_DIR, "modules"), "tests_dir_modules")
+        shutil.copytree(
+            nice_file(TESTS_DIR, "modules"),
+            "tests_dir_modules",
+            ignore=shutil.ignore_patterns("__pycache__"),
+        )
         sys.path.insert(0, abs_file("tests_dir_modules"))
 
     def coverage_usepkgs(self, **kwargs):
@@ -1224,7 +1229,7 @@ class RelativePathTest(CoverageTest):
             config_file=".coveragerc",
             concurrency="multiprocessing",
             data_suffix=False,
-            )
+        )
         cov.start()
         cov.stop()
         # The warning isn't the point of this test, but suppress it.
@@ -1304,7 +1309,7 @@ class CombiningTest(CoverageTest):
         assert_coverage_warnings(
             warns,
             re.compile(
-                r"Couldn't use data file '.*[/\\]\.coverage\.bad': file is not a database"
+                r"Couldn't use data file '.*[/\\]\.coverage\.bad': " + BAD_SQLITE_REGEX
             ),
         )
 
@@ -1335,7 +1340,7 @@ class CombiningTest(CoverageTest):
                 cov.combine(strict=True)
 
         warn_rx = re.compile(
-            r"Couldn't use data file '.*[/\\]\.coverage\.bad[12]': file is not a database"
+            r"Couldn't use data file '.*[/\\]\.coverage\.bad[12]': " + BAD_SQLITE_REGEX
         )
         assert_coverage_warnings(warns, warn_rx, warn_rx)
 
