@@ -6,13 +6,20 @@
 import pytest
 
 from tests.coveragetest import CoverageTest
-from tests.helpers import assert_count_equal
+from tests.helpers import assert_count_equal, xfail_pypy_3749
 
 import coverage
 from coverage import env
 from coverage.files import abs_file
 
 
+skip_cpython_92236 = pytest.mark.skipif(
+    env.PYVERSION == (3, 11, 0, "beta", 1, 0),
+    reason="Avoid a CPython bug: https://github.com/python/cpython/issues/92236",
+    # #92236 is fixed in https://github.com/python/cpython/pull/92722
+    # and in https://github.com/python/cpython/pull/92772
+)
+
 class SimpleArcTest(CoverageTest):
     """Tests for coverage.py's arc measurement."""
 
@@ -135,7 +142,7 @@ class SimpleArcTest(CoverageTest):
             assert foo() == 3 # 7
             """,
             arcz=".1 17 7.  .2 23 36 25 56 6.", arcz_missing="25 56"
-            )
+        )
         self.check_coverage("""\
             def foo():
                 if foo:
@@ -145,12 +152,14 @@ class SimpleArcTest(CoverageTest):
             foo() # 6
             """,
             arcz=".1 16 6.  .2 23 3. 25 5.", arcz_missing="25 5."
-            )
+        )
 
     def test_what_is_the_sound_of_no_lines_clapping(self):
         if env.JYTHON:
             # Jython reports no lines for an empty file.
             arcz_missing=".1 1."                    # pragma: only jython
+        elif env.PYBEHAVIOR.empty_is_empty:
+            arcz_missing=".1 1."
         else:
             # Other Pythons report one line.
             arcz_missing=""
@@ -176,17 +185,16 @@ class SimpleArcTest(CoverageTest):
             """,
             arcz=".1 19 9-1  .2 23 27 34 47 56 67 7-1  9A A9",
             arcz_unpredicted="45",
-            )
+        )
 
 
 class WithTest(CoverageTest):
     """Arc-measuring tests involving context managers."""
 
     def test_with(self):
+        arcz = ".1 .2 23 34 4. 16 6."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 42 2. 16 6."
-        else:
-            arcz = ".1 .2 23 34 4. 16 6."
+            arcz = arcz.replace("4.", "42 2.")
         self.check_coverage("""\
             def example():
                 with open("test", "w") as f:
@@ -196,13 +204,12 @@ class WithTest(CoverageTest):
             example()
             """,
             arcz=arcz,
-            )
+        )
 
     def test_with_return(self):
+        arcz = ".1 .2 23 34 4. 16 6."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 42 2. 16 6."
-        else:
-            arcz = ".1 .2 23 34 4. 16 6."
+            arcz = arcz.replace("4.", "42 2.")
         self.check_coverage("""\
             def example():
                 with open("test", "w") as f:
@@ -212,14 +219,13 @@ class WithTest(CoverageTest):
             example()
             """,
             arcz=arcz,
-            )
+        )
 
     def test_bug_146(self):
         # https://github.com/nedbat/coveragepy/issues/146
+        arcz = ".1 12 23 34 41 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 32 24 41 15 5."
-        else:
-            arcz = ".1 12 23 34 41 15 5."
+            arcz = arcz.replace("34", "32 24")
         self.check_coverage("""\
             for i in range(2):
                 with open("test", "w") as f:
@@ -228,13 +234,12 @@ class WithTest(CoverageTest):
             print(5)
             """,
             arcz=arcz,
-            )
+        )
 
     def test_nested_with_return(self):
+        arcz = ".1 .2 23 34 45 56 6. 18 8."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 .2 23 34 45 56 64 42 2. 18 8."
-        else:
-            arcz = ".1 .2 23 34 45 56 6. 18 8."
+            arcz = arcz.replace("6.", "64 42 2.")
         self.check_coverage("""\
             def example(x):
                 with open("test", "w") as f2:
@@ -246,13 +251,12 @@ class WithTest(CoverageTest):
             example(8)
             """,
             arcz=arcz,
-            )
+        )
 
     def test_break_through_with(self):
+        arcz = ".1 12 23 34 45 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 34 42 25 15 5."
-        else:
-            arcz = ".1 12 23 34 45 15 5."
+            arcz = arcz.replace("45", "42 25")
         self.check_coverage("""\
             for i in range(1+1):
                 with open("test", "w") as f:
@@ -262,13 +266,12 @@ class WithTest(CoverageTest):
             """,
             arcz=arcz,
             arcz_missing="15",
-            )
+        )
 
     def test_continue_through_with(self):
+        arcz = ".1 12 23 34 41 15 5."
         if env.PYBEHAVIOR.exit_through_with:
-            arcz = ".1 12 23 34 42 21 15 5."
-        else:
-            arcz = ".1 12 23 34 41 15 5."
+            arcz = arcz.replace("41", "42 21")
         self.check_coverage("""\
             for i in range(1+1):
                 with open("test", "w") as f:
@@ -277,12 +280,8 @@ class WithTest(CoverageTest):
             print(5)
             """,
             arcz=arcz,
-            )
+        )
 
-    @pytest.mark.skipif(
-        (3, 11) <= env.PYVERSION <= (3, 11, 0, 'alpha', 2, 0),
-        reason="avoid a 3.11 bug: 45709"
-    )
     # https://github.com/nedbat/coveragepy/issues/1270
     def test_raise_through_with(self):
         if env.PYBEHAVIOR.exit_through_with:
@@ -308,15 +307,13 @@ class WithTest(CoverageTest):
             arcz=arcz,
             arcz_missing=arcz_missing,
             arcz_unpredicted=arcz_unpredicted,
-            )
+        )
         expected = "line 3 didn't jump to the function exit"
         assert self.get_missing_arc_description(cov, 3, -2) == expected
 
     def test_untaken_raise_through_with(self):
         if env.PYBEHAVIOR.exit_through_with:
-            #arcz = ".1 12 28 89 9. AB B.  -23 3-2 34 45 56 53 63 37 7-2"
             arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 53 63 37 7-2"
-            #arcz_missing = "3-2 56 63 AB B."
             arcz_missing = "56 63 AB B."
         else:
             arcz = ".1 12 28 89 9. AB B.  -23 34 45 56 6-2 57 7-2"
@@ -336,7 +333,7 @@ class WithTest(CoverageTest):
             """,
             arcz=arcz,
             arcz_missing=arcz_missing,
-            )
+        )
         expected = "line 3 didn't jump to the function exit"
         assert self.get_missing_arc_description(cov, 3, -2) == expected
 
@@ -436,7 +433,7 @@ class LoopArcTest(CoverageTest):
             assert a == 4 and i == 3
             """,
             arcz=arcz,
-            )
+        )
 
     def test_while_true(self):
         # With "while True", 2.x thinks it's computation,
@@ -500,7 +497,7 @@ class LoopArcTest(CoverageTest):
                 break
             """,
             arcz=arcz
-            )
+        )
 
     def test_for_if_else_for(self):
         self.check_coverage("""\
@@ -527,7 +524,7 @@ class LoopArcTest(CoverageTest):
                 ".2 23 34 43 26 3. 6. " +
                 "-89 9A 9-8 AB BC CB B9 AE E9",
             arcz_missing="26 6."
-            )
+        )
 
     def test_for_else(self):
         self.check_coverage("""\
@@ -542,7 +539,7 @@ class LoopArcTest(CoverageTest):
             forelse([1,6])
             """,
             arcz=".1 .2 23 32 34 47 26 67 7. 18 89 9."
-            )
+        )
 
     def test_while_else(self):
         self.check_coverage("""\
@@ -580,7 +577,7 @@ class LoopArcTest(CoverageTest):
         )
 
     # https://bugs.python.org/issue44672
-    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    @pytest.mark.xfail(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
     def test_incorrect_loop_exit_bug_1175(self):
         self.check_coverage("""\
             def wrong_loop(x):
@@ -597,7 +594,7 @@ class LoopArcTest(CoverageTest):
         )
 
     # https://bugs.python.org/issue44672
-    @pytest.mark.skipif(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
+    @pytest.mark.xfail(env.PYVERSION < (3, 10), reason="<3.10 traced final pass incorrectly")
     def test_incorrect_if_bug_1175(self):
         self.check_coverage("""\
             def wrong_loop(x):
@@ -613,6 +610,7 @@ class LoopArcTest(CoverageTest):
             arcz_missing="26 3. 6.",
         )
 
+    @skip_cpython_92236
     def test_generator_expression(self):
         # Generator expression:
         self.check_coverage("""\
@@ -625,6 +623,7 @@ class LoopArcTest(CoverageTest):
             arcz=".1 -22 2-2 12 23 34 45 53 3.",
         )
 
+    @skip_cpython_92236
     def test_generator_expression_another_way(self):
         # https://bugs.python.org/issue44450
         # Generator expression:
@@ -856,10 +855,9 @@ class ExceptionArcTest(CoverageTest):
 
 
     def test_break_through_finally(self):
+        arcz = ".1 12 23 34 3D 45 56 67 68 7A    AD 8A A3 BC CD D."
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 7D 8A A3 A7 BC CD D."
-        else:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 8A A3 AD BC CD D."
+            arcz = arcz.replace("AD", "A7 7D")
         self.check_coverage("""\
             a, c, d, i = 1, 1, 1, 99
             try:
@@ -900,10 +898,9 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_continue_through_finally(self):
+        arcz = ".1 12 23 34 3D 45 56 67 68       7A 8A A3 BC CD D."
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 23 34 3D 45 56 67 68 73 7A 8A A3 A7 BC CD D."
-        else:
-            arcz = ".1 12 23 34 3D 45 56 67 68 7A 8A A3 BC CD D."
+            arcz += " 73 A7"
         self.check_coverage("""\
             a, b, c, d, i = 1, 1, 1, 1, 99
             try:
@@ -1059,10 +1056,9 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_return_finally(self):
+        arcz = ".1 12 29 9A AB BC C-1   -23 34 45  7-2   57 38 8-2"
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = ".1 12 29 9A AB BC C-1   -23 34 45 5-2 57 75 38 8-2"
-        else:
-            arcz = ".1 12 29 9A AB BC C-1   -23 34 45 57 7-2 38 8-2"
+            arcz = arcz.replace("7-2", "75 5-2")
         self.check_coverage("""\
             a = [1]
             def check_token(data):
@@ -1081,22 +1077,15 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_except_jump_finally(self):
+        arcz = (
+            ".1 1Q QR RS ST TU U. " +
+            ".2 23 34 45 56 4O 6L " +
+            "78 89 9A AL   8B BC CD DL   BE EF FG GL   EH HI IJ JL  HL " +
+            "LO L4 L. LM " +
+            "MN NO O."
+        )
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = (
-                ".1 1Q QR RS ST TU U. " +
-                ".2 23 34 45 56 4O 6L " +
-                "78 89 9A AL LA AO  8B BC CD DL LD D4  BE EF FG GL LG G.  EH HI IJ JL  HL " +
-                "L4 LM " +
-                "MN NO O."
-            )
-        else:
-            arcz = (
-                ".1 1Q QR RS ST TU U. " +
-                ".2 23 34 45 56 4O 6L " +
-                "78 89 9A AL  8B BC CD DL  BE EF FG GL  EH HI IJ JL  HL " +
-                "LO L4 L. LM " +
-                "MN NO O."
-            )
+            arcz = arcz.replace("LO", "LA AO").replace("L4", "L4 LD D4").replace("L.", "LG G.")
         self.check_coverage("""\
             def func(x):
                 a = f = g = 2
@@ -1135,22 +1124,15 @@ class ExceptionArcTest(CoverageTest):
         )
 
     def test_else_jump_finally(self):
+        arcz = (
+            ".1 1S ST TU UV VW W. " +
+            ".2 23 34 45 56 6A 78 8N 4Q " +
+            "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN " +
+            "N4 NQ N. NO " +
+            "OP PQ Q."
+        )
         if env.PYBEHAVIOR.finally_jumps_back:
-            arcz = (
-                ".1 1S ST TU UV VW W. " +
-                ".2 23 34 45 56 6A 78 8N 4Q " +
-                "AB BC CN NC CQ  AD DE EF FN NF F4  DG GH HI IN NI I.  GJ JK KL LN  JN " +
-                "N4 NO " +
-                "OP PQ Q."
-            )
-        else:
-            arcz = (
-                ".1 1S ST TU UV VW W. " +
-                ".2 23 34 45 56 6A 78 8N 4Q " +
-                "AB BC CN  AD DE EF FN  DG GH HI IN  GJ JK KL LN  JN " +
-                "N4 NQ N. NO " +
-                "OP PQ Q."
-            )
+            arcz = arcz.replace("NQ", "NC CQ").replace("N4", "N4 NF F4").replace("N.", "NI I.")
         self.check_coverage("""\
             def func(x):
                 a = f = g = 2
@@ -1194,6 +1176,7 @@ class ExceptionArcTest(CoverageTest):
 class YieldTest(CoverageTest):
     """Arc tests for generators."""
 
+    @skip_cpython_92236
     def test_yield_in_loop(self):
         self.check_coverage("""\
             def gen(inp):
@@ -1205,6 +1188,7 @@ class YieldTest(CoverageTest):
             arcz=".1 .2 23 2. 32 15 5.",
         )
 
+    @skip_cpython_92236
     def test_padded_yield_in_loop(self):
         self.check_coverage("""\
             def gen(inp):
@@ -1220,6 +1204,7 @@ class YieldTest(CoverageTest):
             arcz=".1 19 9.  .2 23 34 45 56 63 37 7.",
         )
 
+    @skip_cpython_92236
     def test_bug_308(self):
         self.check_coverage("""\
             def run():
@@ -1254,6 +1239,7 @@ class YieldTest(CoverageTest):
             arcz=".1 14 45 54 4.  .2 2.  -22 2-2",
         )
 
+    @skip_cpython_92236
     def test_bug_324(self):
         # This code is tricky: the list() call pulls all the values from gen(),
         # but each of them is a generator itself that is never iterated.  As a
@@ -1272,6 +1258,7 @@ class YieldTest(CoverageTest):
             arcz_missing="-33 3-3",
         )
 
+    @skip_cpython_92236
     def test_coroutines(self):
         self.check_coverage("""\
             def double_inputs():
@@ -1291,6 +1278,7 @@ class YieldTest(CoverageTest):
         )
         assert self.stdout() == "20\n12\n"
 
+    @skip_cpython_92236
     def test_yield_from(self):
         self.check_coverage("""\
             def gen(inp):
@@ -1304,9 +1292,9 @@ class YieldTest(CoverageTest):
             list(gen([1,2,3]))
             """,
             arcz=".1 19 9.  .2 23 34 45 56 63 37 7.",
-            arcz_unpredicted="5.",
         )
 
+    @skip_cpython_92236
     def test_abandoned_yield(self):
         # https://github.com/nedbat/coveragepy/issues/440
         self.check_coverage("""\
@@ -1324,7 +1312,6 @@ class YieldTest(CoverageTest):
         )
 
 
-@pytest.mark.skipif(env.PYVERSION <= (3, 10, 0, 'beta', 4, 0), reason="3.10.0b4 had bugs")
 @pytest.mark.skipif(not env.PYBEHAVIOR.match_case, reason="Match-case is new in 3.10")
 class MatchCaseTest(CoverageTest):
     """Tests of match-case."""
@@ -1532,18 +1519,14 @@ class OptimizedIfTest(CoverageTest):
         )
 
     def test_if_not_debug(self):
-        arcz_missing = ""
-        if env.PYBEHAVIOR.pep626:
+        if env.PYBEHAVIOR.optimize_if_not_debug == 1:
             arcz = ".1 12 23 34 42 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug3:
-            arcz = ".1 12 23 32 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug2:
+        elif env.PYBEHAVIOR.optimize_if_not_debug == 2:
             arcz = ".1 12 23 35 52 37 72 28 8."
-        elif env.PYBEHAVIOR.optimize_if_not_debug:
-            arcz = ".1 12 23 34 42 37 72 28 8."
         else:
-            arcz = ".1 12 23 34 45 42 52 37 72 28 8."
-            arcz_missing = "45 52"
+            assert env.PYBEHAVIOR.optimize_if_not_debug == 3
+            arcz = ".1 12 23 32 37 72 28 8."
+
         self.check_coverage("""\
             lines = set()
             for value in [True, False]:
@@ -1555,7 +1538,6 @@ class OptimizedIfTest(CoverageTest):
             assert lines == set([7])
             """,
             arcz=arcz,
-            arcz_missing=arcz_missing,
         )
 
 
@@ -1640,6 +1622,7 @@ class MiscArcTest(CoverageTest):
         self.check_coverage(code, arcs=[(-1, 1), (1, 2*n+4), (2*n+4, -1)])
         assert self.stdout() == f"{n}\n"
 
+    @skip_cpython_92236
     def test_partial_generators(self):
         # https://github.com/nedbat/coveragepy/issues/475
         # Line 2 is executed completely.
@@ -1666,7 +1649,15 @@ class MiscArcTest(CoverageTest):
 class DecoratorArcTest(CoverageTest):
     """Tests of arcs with decorators."""
 
+    @xfail_pypy_3749
     def test_function_decorator(self):
+        arcz = (
+            ".1 16 67 7A AE EF F. "     # main line
+            ".2 24 4.   -23 3-2 "       # decorators
+            "-6D D-6 "                  # my_function
+        )
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "A7 76 6A "
         self.check_coverage("""\
             def decorator(arg):
                 def _dec(f):
@@ -1684,13 +1675,18 @@ class DecoratorArcTest(CoverageTest):
             a = 14
             my_function()
             """,
-            arcz=
-                ".1 16 67 7A AE EF F. "     # main line
-                ".2 24 4.   -23 3-2 "       # decorators
-                "-6D D-6 ",                 # my_function
+            arcz=arcz,
         )
 
+    @xfail_pypy_3749
     def test_class_decorator(self):
+        arcz = (
+            ".1 16 67 6D 7A AE E. "     # main line
+            ".2 24 4.   -23 3-2 "       # decorators
+            "-66 D-6 "                  # MyObject
+        )
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "A7 76 6A "
         self.check_coverage("""\
             def decorator(arg):
                 def _dec(c):
@@ -1707,19 +1703,19 @@ class DecoratorArcTest(CoverageTest):
                 X = 13
             a = 14
             """,
-            arcz=
-                ".1 16 67 6D 7A AE E. "     # main line
-                ".2 24 4.   -23 3-2 "       # decorators
-                "-66 D-6 ",                 # MyObject
+            arcz=arcz,
         )
 
-    def test_bug_466(self):
+    @xfail_pypy_3749
+    def test_bug_466a(self):
         # A bad interaction between decorators and multi-line list assignments,
         # believe it or not...!
+        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 1A A.  13 34 4.  -35 58 8-3"
-        else:
-            arcz = ".1 1A A.  13 3.     -35 58 8-3"
+            arcz = arcz.replace("3.", "34 4.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "43 "
+        # This example makes more sense when considered in tandem with 466b below.
         self.check_coverage("""\
             class Parser(object):
 
@@ -1734,10 +1730,16 @@ class DecoratorArcTest(CoverageTest):
             """,
             arcz=arcz,
         )
+
+    @xfail_pypy_3749
+    def test_bug_466b(self):
+        # A bad interaction between decorators and multi-line list assignments,
+        # believe it or not...!
+        arcz = ".1 1A A.  13 3.     -35 58 8-3 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 1A A.  13 34 4.  -35 58 8-3"
-        else:
-            arcz = ".1 1A A.  13 3.     -35 58 8-3"
+            arcz = arcz.replace("3.", "34 4.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "43 "
         self.check_coverage("""\
             class Parser(object):
 
@@ -1788,7 +1790,7 @@ class LambdaArcTest(CoverageTest):
             b = 3
             """,
             arcz=".1 12 -22 2-2 23 3.", arcz_missing="-22 2-2",
-            )
+        )
 
     def test_raise_with_lambda_looks_like_partial_branch(self):
         self.check_coverage("""\
@@ -1830,8 +1832,8 @@ class LambdaArcTest(CoverageTest):
         )
 
 
-skip_eventlet_670 = pytest.mark.skipif(
-    env.PYVERSION[:2] == (3, 9) and env.OSX,
+xfail_eventlet_670 = pytest.mark.xfail(
+    env.PYVERSION[:2] == (3, 9) and env.CPYTHON and env.OSX,
     reason="Avoid an eventlet bug on Mac 3.9: eventlet#670",
     # https://github.com/eventlet/eventlet/issues/670
 )
@@ -1840,7 +1842,8 @@ skip_eventlet_670 = pytest.mark.skipif(
 class AsyncTest(CoverageTest):
     """Tests of the new async and await keywords in Python 3.5"""
 
-    @skip_eventlet_670
+    @xfail_eventlet_670
+    @skip_cpython_92236
     def test_async(self):
         self.check_coverage("""\
             import asyncio
@@ -1864,11 +1867,11 @@ class AsyncTest(CoverageTest):
                 ".1 13 38 8E EF FG G. " +
                 "-34 45 56 6-3 " +
                 "-89 9C C-8",
-            arcz_unpredicted="5-3 9-8",
         )
         assert self.stdout() == "Compute 1 + 2 ...\n1 + 2 = 3\n"
 
-    @skip_eventlet_670
+    @xfail_eventlet_670
+    @skip_cpython_92236
     def test_async_for(self):
         self.check_coverage("""\
             import asyncio
@@ -1922,11 +1925,13 @@ class AsyncTest(CoverageTest):
             arcz_missing=arcz_missing,
         )
 
+    @xfail_pypy_3749
     def test_async_decorator(self):
+        arcz = ".1 14 4.     .2 2.  -46 6-4 "
         if env.PYBEHAVIOR.trace_decorated_def:
-            arcz = ".1 14 45 5.  .2 2.  -46 6-4"
-        else:
-            arcz = ".1 14 4.     .2 2.  -46 6-4"
+            arcz = arcz.replace("4.", "45 5.")
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            arcz += "54 "
         self.check_coverage("""\
             def wrap(f):        # 1
                 return f
@@ -1942,8 +1947,8 @@ class AsyncTest(CoverageTest):
     # https://github.com/nedbat/coveragepy/issues/1158
     # https://bugs.python.org/issue44621
     @pytest.mark.skipif(env.PYVERSION[:2] == (3, 9), reason="avoid a 3.9 bug: 44621")
-    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
-    def test_bug1158(self):
+    @skip_cpython_92236
+    def test_bug_1158(self):
         self.check_coverage("""\
             import asyncio
 
@@ -1967,13 +1972,9 @@ class AsyncTest(CoverageTest):
 
     # https://github.com/nedbat/coveragepy/issues/1176
     # https://bugs.python.org/issue44622
-    @pytest.mark.skipif(
-        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "beta", 4, 0),
-        reason="avoid a 3.10 bug fixed after beta 4: 44622"
-    )
-    @skip_eventlet_670
-    @pytest.mark.skipif(env.PYVERSION < (3, 7), reason="need asyncio.run")
-    def test_bug1176(self):
+    @xfail_eventlet_670
+    @skip_cpython_92236
+    def test_bug_1176(self):
         self.check_coverage("""\
             import asyncio
 
@@ -1991,12 +1992,7 @@ class AsyncTest(CoverageTest):
         assert self.stdout() == "12\n"
 
     # https://github.com/nedbat/coveragepy/issues/1205
-    # https://bugs.python.org/issue44840
-    @pytest.mark.skipif(
-        (3, 10, 0, "alpha", 0, 0) <= env.PYVERSION[:6] <= (3, 10, 0, "candidate", 1, 0),
-        reason="avoid a 3.10 bug fixed after rc1: 44840"
-    )
-    def test_bug1205(self):
+    def test_bug_1205(self):
         self.check_coverage("""\
             def func():
                 if T(2):
