@@ -22,7 +22,6 @@ from coverage.files import abs_file
 from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
-from tests.helpers import remove_files
 
 
 # These libraries aren't always available, we'll skip tests if they aren't.
@@ -353,31 +352,27 @@ class ConcurrencyTest(CoverageTest):
         assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
     def test_bad_concurrency(self):
-        self.make_file("prog.py", "a = 1")
-        msg = "Unknown concurrency choices: nothing"
-        with pytest.raises(ConfigError, match=msg):
+        with pytest.raises(ConfigError, match="Unknown concurrency choices: nothing"):
             self.command_line("run --concurrency=nothing prog.py")
 
     def test_bad_concurrency_in_config(self):
-        self.make_file("prog.py", "a = 1")
         self.make_file(".coveragerc", "[run]\nconcurrency = nothing\n")
-        msg = "Unknown concurrency choices: nothing"
-        with pytest.raises(ConfigError, match=msg):
+        with pytest.raises(ConfigError, match="Unknown concurrency choices: nothing"):
             self.command_line("run prog.py")
 
     def test_no_multiple_light_concurrency(self):
-        self.make_file("prog.py", "a = 1")
-        msg = "Conflicting concurrency settings: eventlet, gevent"
-        with pytest.raises(ConfigError, match=msg):
+        with pytest.raises(ConfigError, match="Conflicting concurrency settings: eventlet, gevent"):
             self.command_line("run --concurrency=gevent,eventlet prog.py")
 
     def test_no_multiple_light_concurrency_in_config(self):
-        self.make_file("prog.py", "a = 1")
         self.make_file(".coveragerc", "[run]\nconcurrency = gevent, eventlet\n")
-        msg = "Conflicting concurrency settings: eventlet, gevent"
-        with pytest.raises(ConfigError, match=msg):
+        with pytest.raises(ConfigError, match="Conflicting concurrency settings: eventlet, gevent"):
             self.command_line("run prog.py")
 
+    def test_multiprocessing_needs_config_file(self):
+        with pytest.raises(ConfigError, match="multiprocessing requires a configuration file"):
+            self.command_line("run --concurrency=multiprocessing prog.py")
+
 
 class WithoutConcurrencyModuleTest(CoverageTest):
     """Tests of what happens if the requested concurrency isn't installed."""
@@ -438,13 +433,30 @@ MULTI_CODE = """
     """
 
 
+@pytest.fixture(params=["fork", "spawn"], name="start_method")
+def start_method_fixture(request):
+    """Parameterized fixture to choose the start_method for multiprocessing."""
+    start_method = request.param
+    if start_method not in multiprocessing.get_all_start_methods():
+        # Windows doesn't support "fork".
+        pytest.skip(f"start_method={start_method} not supported here")
+    return start_method
+
+
 @pytest.mark.skipif(not multiprocessing, reason="No multiprocessing in this Python")
 @flaky(max_runs=30)         # Sometimes a test fails due to inherent randomness. Try more times.
 class MultiprocessingTest(CoverageTest):
     """Test support of the multiprocessing module."""
 
     def try_multiprocessing_code(
-        self, code, expected_out, the_module, nprocs, concurrency="multiprocessing", args=""
+        self,
+        code,
+        expected_out,
+        the_module,
+        nprocs,
+        start_method,
+        concurrency="multiprocessing",
+        args="",
     ):
         """Run code using multiprocessing, it should produce `expected_out`."""
         self.make_file("multi.py", code)
@@ -454,54 +466,62 @@ class MultiprocessingTest(CoverageTest):
             source = .
             """)
 
-        for start_method in ["fork", "spawn"]:
-            if start_method and start_method not in multiprocessing.get_all_start_methods():
-                continue
+        cmd = "coverage run {args} multi.py {start_method}".format(
+            args=args, start_method=start_method,
+        )
+        out = self.run_command(cmd)
+        expected_cant_trace = cant_trace_msg(concurrency, the_module)
 
-            remove_files(".coverage", ".coverage.*")
-            cmd = "coverage run {args} multi.py {start_method}".format(
-                args=args, start_method=start_method,
+        if expected_cant_trace is not None:
+            print(out)
+            assert out == expected_cant_trace
+            pytest.skip(f"Can't test: {expected_cant_trace}")
+        else:
+            assert out.rstrip() == expected_out
+            assert len(glob.glob(".coverage.*")) == nprocs + 1
+
+            out = self.run_command("coverage combine")
+            out_lines = out.splitlines()
+            assert len(out_lines) == nprocs + 1
+            assert all(
+                re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
+                for line in out_lines
             )
-            out = self.run_command(cmd)
-            expected_cant_trace = cant_trace_msg(concurrency, the_module)
+            out = self.run_command("coverage report -m")
 
-            if expected_cant_trace is not None:
-                print(out)
-                assert out == expected_cant_trace
-                pytest.skip(f"Can't test: {expected_cant_trace}")
-            else:
-                assert out.rstrip() == expected_out
-                assert len(glob.glob(".coverage.*")) == nprocs + 1
+            last_line = self.squeezed_lines(out)[-1]
+            assert re.search(r"TOTAL \d+ 0 100%", last_line)
 
-                out = self.run_command("coverage combine")
-                out_lines = out.splitlines()
-                assert len(out_lines) == nprocs + 1
-                assert all(
-                    re.fullmatch(r"Combined data file \.coverage\..*\.\d+\.\d+", line)
-                    for line in out_lines
-                )
-                out = self.run_command("coverage report -m")
-
-                last_line = self.squeezed_lines(out)[-1]
-                assert re.search(r"TOTAL \d+ 0 100%", last_line)
-
-    def test_multiprocessing_simple(self):
+    def test_multiprocessing_simple(self, start_method):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
-        self.try_multiprocessing_code(code, expected_out, threading, nprocs)
+        self.try_multiprocessing_code(
+            code,
+            expected_out,
+            threading,
+            nprocs,
+            start_method=start_method,
+        )
 
-    def test_multiprocessing_append(self):
+    def test_multiprocessing_append(self, start_method):
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
         total = sum(x*x if x%2 else x*x*x for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
-        self.try_multiprocessing_code(code, expected_out, threading, nprocs, args="--append")
+        self.try_multiprocessing_code(
+            code,
+            expected_out,
+            threading,
+            nprocs,
+            args="--append",
+            start_method=start_method,
+        )
 
-    def test_multiprocessing_and_gevent(self):
+    def test_multiprocessing_and_gevent(self, start_method):
         nprocs = 3
         upto = 30
         code = (
@@ -510,14 +530,15 @@ class MultiprocessingTest(CoverageTest):
         total = sum(sum(range((x + 1) * 100)) for x in range(upto))
         expected_out = f"{nprocs} pids, total = {total}"
         self.try_multiprocessing_code(
-            code, expected_out, eventlet, nprocs, concurrency="multiprocessing,eventlet"
+            code,
+            expected_out,
+            eventlet,
+            nprocs,
+            concurrency="multiprocessing,eventlet",
+            start_method=start_method,
         )
 
-    @pytest.mark.parametrize("start_method", ["fork", "spawn"])
     def test_multiprocessing_with_branching(self, start_method):
-        if start_method not in multiprocessing.get_all_start_methods():
-            pytest.skip(f"{start_method} not supported here")
-
         nprocs = 3
         upto = 30
         code = (SQUARE_OR_CUBE_WORK + MULTI_CODE).format(NPROCS=nprocs, UPTO=upto)
@@ -558,7 +579,7 @@ class MultiprocessingTest(CoverageTest):
         assert "Exception during multiprocessing bootstrap init" in out
         assert "Exception: Crashing because called by _bootstrap" in out
 
-    def test_bug890(self):
+    def test_bug_890(self):
         # chdir in multiprocessing shouldn't keep us from finding the
         # .coveragerc file.
         self.make_file("multi.py", """\
@@ -668,3 +689,90 @@ def test_thread_safe_save_data(tmpdir):
     finally:
         os.chdir(old_dir)
         should_run[0] = False
+
+
+@pytest.mark.skipif(env.WINDOWS, reason="SIGTERM doesn't work the same on Windows")
+@flaky(max_runs=3)          # Sometimes a test fails due to inherent randomness. Try more times.
+class SigtermTest(CoverageTest):
+    """Tests of our handling of SIGTERM."""
+
+    @pytest.mark.parametrize("sigterm", [False, True])
+    def test_sigterm_saves_data(self, sigterm):
+        # A terminated process should save its coverage data.
+        self.make_file("clobbered.py", """\
+            import multiprocessing
+            import time
+
+            def subproc(x):
+                if x.value == 3:
+                    print("THREE", flush=True)  # line 6, missed
+                else:
+                    print("NOT THREE", flush=True)
+                x.value = 0
+                time.sleep(60)
+
+            if __name__ == "__main__":
+                print("START", flush=True)
+                x = multiprocessing.Value("L", 1)
+                proc = multiprocessing.Process(target=subproc, args=(x,))
+                proc.start()
+                while x.value != 0:
+                    time.sleep(.05)
+                proc.terminate()
+                print("END", flush=True)
+            """)
+        self.make_file(".coveragerc", """\
+            [run]
+            parallel = True
+            concurrency = multiprocessing
+            """ + ("sigterm = true" if sigterm else "")
+            )
+        out = self.run_command("coverage run clobbered.py")
+        # Under the Python tracer on Linux, we get the "Trace function changed"
+        # message. Does that matter?
+        if "Trace function changed" in out:
+            lines = out.splitlines(True)
+            assert len(lines) == 5  # "trace function changed" and "self.warn("
+            out = "".join(lines[:3])
+        assert out == "START\nNOT THREE\nEND\n"
+        self.run_command("coverage combine")
+        out = self.run_command("coverage report -m")
+        if sigterm:
+            expected = "clobbered.py 17 1 94% 6"
+        else:
+            expected = "clobbered.py 17 5 71% 5-10"
+        assert self.squeezed_lines(out)[2] == expected
+
+    def test_sigterm_still_runs(self):
+        # A terminated process still runs its own SIGTERM handler.
+        self.make_file("handler.py", """\
+            import multiprocessing
+            import signal
+            import time
+
+            def subproc(x):
+                print("START", flush=True)
+                def on_sigterm(signum, frame):
+                    print("SIGTERM", flush=True)
+
+                signal.signal(signal.SIGTERM, on_sigterm)
+                x.value = 0
+                time.sleep(.1)
+                print("END", flush=True)
+
+            if __name__ == "__main__":
+                x = multiprocessing.Value("L", 1)
+                proc = multiprocessing.Process(target=subproc, args=(x,))
+                proc.start()
+                while x.value != 0:
+                    time.sleep(.02)
+                proc.terminate()
+            """)
+        self.make_file(".coveragerc", """\
+            [run]
+            parallel = True
+            concurrency = multiprocessing
+            sigterm = True
+            """)
+        out = self.run_command("coverage run handler.py")
+        assert out == "START\nSIGTERM\nEND\n"
