@@ -10,6 +10,7 @@ from coverage import env
 from coverage.exceptions import NoDataError
 
 from tests.coveragetest import CoverageTest
+from tests.helpers import xfail_pypy_3749
 
 
 class TestCoverageTest(CoverageTest):
@@ -22,14 +23,14 @@ class TestCoverageTest(CoverageTest):
             b = 2
             """,
             [1,2]
-            )
+        )
         # You can provide a list of possible statement matches.
         self.check_coverage("""\
             a = 1
             b = 2
             """,
             ([100], [1,2], [1723,47]),
-            )
+        )
         # You can specify missing lines.
         self.check_coverage("""\
             a = 1
@@ -38,7 +39,7 @@ class TestCoverageTest(CoverageTest):
             """,
             [1,2,3],
             missing="3",
-            )
+        )
         # You can specify a list of possible missing lines.
         self.check_coverage("""\
             a = 1
@@ -47,7 +48,7 @@ class TestCoverageTest(CoverageTest):
             """,
             [1,2,3],
             missing=("47-49", "3", "100,102")
-            )
+        )
 
     def test_failed_coverage(self):
         # If the lines are wrong, the message shows right and wrong.
@@ -96,7 +97,7 @@ class TestCoverageTest(CoverageTest):
                 a = 1
                 assert a == 99, "This is bad"
                 """
-                )
+            )
         # Other exceptions too.
         with pytest.raises(ZeroDivisionError, match="division"):
             self.check_coverage("""\
@@ -104,7 +105,7 @@ class TestCoverageTest(CoverageTest):
                 assert a == 1, "This is good"
                 a/0
                 """
-                )
+            )
 
 
 class BasicCoverageTest(CoverageTest):
@@ -1251,7 +1252,7 @@ class ExcludeTest(CoverageTest):
             g = 7
             """,
             [1,3,5,7]
-            )
+        )
 
     def test_simple(self):
         self.check_coverage("""\
@@ -1617,6 +1618,7 @@ class ExcludeTest(CoverageTest):
 class Py24Test(CoverageTest):
     """Tests of new syntax in Python 2.4."""
 
+    @xfail_pypy_3749
     def test_function_decorators(self):
         lines = [1, 2, 3, 4, 6, 8, 10, 12]
         if env.PYBEHAVIOR.trace_decorated_def:
@@ -1637,6 +1639,7 @@ class Py24Test(CoverageTest):
             """,
             lines, "")
 
+    @xfail_pypy_3749
     def test_function_decorators_with_args(self):
         lines = [1, 2, 3, 4, 5, 6, 8, 10, 12]
         if env.PYBEHAVIOR.trace_decorated_def:
@@ -1657,6 +1660,7 @@ class Py24Test(CoverageTest):
             """,
             lines, "")
 
+    @xfail_pypy_3749
     def test_double_function_decorators(self):
         lines = [1, 2, 3, 4, 5, 7, 8, 9, 10, 11, 12, 14, 15, 17, 19, 21, 22, 24, 26]
         if env.PYBEHAVIOR.trace_decorated_def:
