@@ -67,7 +67,12 @@ def DebugCoverageData(*args, **kwargs):
     lines in our coverage reports.
     """
     assert "debug" not in kwargs
-    debug = DebugControlString(options=["dataio", "dataop", "sql"])
+    options = ["dataio", "dataop", "sql"]
+    if kwargs:
+        # There's no reason kwargs should imply sqldata debugging.
+        # This is a way to get a mix of debug options across the tests.
+        options.extend(["sqldata"])
+    debug = DebugControlString(options=options)
     return CoverageData(*args, debug=debug, **kwargs)
 
 
@@ -113,6 +118,13 @@ class CoverageDataTest(CoverageTest):
     def test_empty_data_is_false(self):
         covdata = DebugCoverageData()
         assert not covdata
+        self.assert_doesnt_exist(".coverage")
+
+    def test_empty_data_is_false_when_read(self):
+        covdata = DebugCoverageData()
+        covdata.read()
+        assert not covdata
+        self.assert_doesnt_exist(".coverage")
 
     def test_line_data_is_true(self):
         covdata = DebugCoverageData()
@@ -160,15 +172,17 @@ class CoverageDataTest(CoverageTest):
         assert_line_counts(covdata, SUMMARY_3_4)
         assert_measured_files(covdata, MEASURED_FILES_3_4)
 
-    def test_cant_add_arcs_with_lines(self):
-        covdata = DebugCoverageData()
+    @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
+    def test_cant_add_arcs_with_lines(self, klass):
+        covdata = klass()
         covdata.add_lines(LINES_1)
         msg = "Can't add branch measurements to existing line data"
         with pytest.raises(DataError, match=msg):
             covdata.add_arcs(ARCS_3)
 
-    def test_cant_add_lines_with_arcs(self):
-        covdata = DebugCoverageData()
+    @pytest.mark.parametrize("klass", [CoverageData, DebugCoverageData])
+    def test_cant_add_lines_with_arcs(self, klass):
+        covdata = klass()
         covdata.add_arcs(ARCS_3)
         msg = "Can't add line measurements to existing branch data"
         with pytest.raises(DataError, match=msg):
@@ -559,7 +573,7 @@ class CoverageDataTest(CoverageTest):
             t.join()
 
         assert_lines1_data(covdata)
-        assert exceptions == []
+        assert not exceptions
 
 
 class CoverageDataInTempDirTest(CoverageTest):
@@ -584,24 +598,16 @@ class CoverageDataInTempDirTest(CoverageTest):
         assert_arcs3_data(covdata2)
 
     def test_read_errors(self):
-        msg = r"Couldn't .* '.*[/\\]{}': \S+"
-
         self.make_file("xyzzy.dat", "xyzzy")
-        with pytest.raises(DataError, match=msg.format("xyzzy.dat")):
+        with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]xyzzy.dat': \S+"):
             covdata = DebugCoverageData("xyzzy.dat")
             covdata.read()
         assert not covdata
 
-        self.make_file("empty.dat", "")
-        with pytest.raises(DataError, match=msg.format("empty.dat")):
-            covdata = DebugCoverageData("empty.dat")
-            covdata.read()
-        assert not covdata
-
     def test_hard_read_error(self):
         self.make_file("noperms.dat", "go away")
         os.chmod("noperms.dat", 0)
-        with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]noperms.dat': "):
+        with pytest.raises(DataError, match=r"Couldn't .* '.*[/\\]noperms.dat': \S+"):
             covdata = DebugCoverageData("noperms.dat")
             covdata.read()
 
@@ -616,7 +622,7 @@ class CoverageDataInTempDirTest(CoverageTest):
             sqldb.close = lambda: 1/0
             covdata.add_lines(LINES_1)
 
-    def test_read_sql_errors(self):
+    def test_wrong_schema_version(self):
         with sqlite3.connect("wrong_schema.db") as con:
             con.execute("create table coverage_schema (version integer)")
             con.execute("insert into coverage_schema (version) values (99)")
@@ -626,11 +632,13 @@ class CoverageDataInTempDirTest(CoverageTest):
             covdata.read()
         assert not covdata
 
-        with sqlite3.connect("no_schema.db") as con:
-            con.execute("create table foobar (baz text)")
-        msg = r"Couldn't .* '.*[/\\]no_schema.db': \S+"
+    def test_wrong_schema_schema(self):
+        with sqlite3.connect("wrong_schema_schema.db") as con:
+            con.execute("create table coverage_schema (xyzzy integer)")
+            con.execute("insert into coverage_schema (xyzzy) values (99)")
+        msg = r"Data file .* doesn't seem to be a coverage data file: .* no such column"
         with pytest.raises(DataError, match=msg):
-            covdata = DebugCoverageData("no_schema.db")
+            covdata = DebugCoverageData("wrong_schema_schema.db")
             covdata.read()
         assert not covdata
 
@@ -667,7 +675,8 @@ class CoverageDataFilesTest(CoverageTest):
 
         assert re.search(
             r"^Erasing data file '.*\.coverage'\n" +
-            r"Creating data file '.*\.coverage'\n" +
+            r"Opening data file '.*\.coverage'\n" +
+            r"Initing data file '.*\.coverage'\n" +
             r"Opening data file '.*\.coverage'\n$",
             debug.get_output()
         )
@@ -869,6 +878,33 @@ class CoverageDataFilesTest(CoverageTest):
         # "no such table: meta"
         covdata2.add_lines(LINES_1)
 
+    @pytest.mark.parametrize(
+        "dpart, fpart",
+        [
+            ("", "[b-a]"),
+            ("[3-1]", ""),
+            ("[3-1]", "[b-a]"),
+        ],
+    )
+    def test_combining_with_crazy_filename(self, dpart, fpart):
+        dirname = f"py{dpart}"
+        basename = f"{dirname}/.coverage{fpart}"
+        os.makedirs(dirname)
+
+        covdata1 = CoverageData(basename=basename, suffix="1")
+        covdata1.add_lines(LINES_1)
+        covdata1.write()
+
+        covdata2 = CoverageData(basename=basename, suffix="2")
+        covdata2.add_lines(LINES_2)
+        covdata2.write()
+
+        covdata3 = CoverageData(basename=basename)
+        combine_parallel_data(covdata3)
+        assert_line_counts(covdata3, SUMMARY_1_2)
+        assert_measured_files(covdata3, MEASURED_FILES_1_2)
+        self.assert_file_count(glob.escape(basename) + ".*", 0)
+
 
 class DumpsLoadsTest(CoverageTest):
     """Tests of CoverageData.dumps and loads."""
@@ -893,6 +929,22 @@ class DumpsLoadsTest(CoverageTest):
         msg = r"Unrecognized serialization: {} \(head of {} bytes\)".format(
             re.escape(repr(bad_data[:40])),
             len(bad_data),
-            )
+        )
         with pytest.raises(DataError, match=msg):
             covdata.loads(bad_data)
+
+
+class NoDiskTest(CoverageTest):
+    """Tests of in-memory CoverageData."""
+
+    run_in_temp_dir = False
+
+    def test_updating(self):
+        # https://github.com/nedbat/coveragepy/issues/1323
+        a = CoverageData(no_disk=True)
+        a.add_lines({'foo.py': [10, 20, 30]})
+        assert a.measured_files() == {'foo.py'}
+
+        b = CoverageData(no_disk=True)
+        b.update(a)
+        assert b.measured_files() == {'foo.py'}
