@@ -149,9 +149,6 @@ class RunFileTest(CoverageTest):
         assert self.stdout() == "about to exit..\n"
         assert self.stderr() == ""
 
-    @pytest.mark.skipif(not env.CPYTHON,
-        reason="non-CPython handles excepthook exits differently, punt for now."
-    )
     def test_excepthook_exit(self):
         self.make_file("excepthook_exit.py", """\
             import sys
@@ -169,7 +166,6 @@ class RunFileTest(CoverageTest):
         cov_out = self.stdout()
         assert cov_out == "in excepthook\n"
 
-    @pytest.mark.skipif(env.PYPY, reason="PyPy handles excepthook throws differently.")
     def test_excepthook_throw(self):
         self.make_file("excepthook_throw.py", """\
             import sys
@@ -242,7 +238,6 @@ class RunPycFileTest(CoverageTest):
         # In some environments, the pycfile persists and pollutes another test.
         os.remove(pycfile)
 
-    @pytest.mark.skipif(not env.PYBEHAVIOR.hashed_pyc_pep552, reason="No hashed .pyc here")
     def test_running_hashed_pyc(self):
         pycfile = self.make_pyc(invalidation_mode=py_compile.PycInvalidationMode.CHECKED_HASH)
         run_python_file([pycfile])
