@@ -3,6 +3,7 @@
 
 """Tests that HTML generation is awesome."""
 
+import collections
 import datetime
 import glob
 import json
@@ -52,7 +53,9 @@ class HtmlTestHelpers(CoverageTest):
         self.clean_local_file_imports()
         cov = coverage.Coverage(**(covargs or {}))
         self.start_import_stop(cov, "main_file")
-        return cov.html_report(**(htmlargs or {}))
+        ret = cov.html_report(**(htmlargs or {}))
+        self.assert_valid_hrefs()
+        return ret
 
     def get_html_report_content(self, module):
         """Return the content of the HTML report for `module`."""
@@ -95,6 +98,29 @@ class HtmlTestHelpers(CoverageTest):
             msg=f"Timestamp is wrong: {timestamp}",
         )
 
+    def assert_valid_hrefs(self):
+        """Assert that the hrefs in htmlcov/*.html to see the references are valid.
+
+        Doesn't check external links (those with a protocol).
+        """
+        hrefs = collections.defaultdict(set)
+        for fname in glob.glob("htmlcov/*.html"):
+            with open(fname) as fhtml:
+                html = fhtml.read()
+            for href in re.findall(r""" href=['"]([^'"]*)['"]""", html):
+                if href.startswith("#"):
+                    assert re.search(rf""" id=['"]{href[1:]}['"]""", html), (
+                        f"Fragment {href!r} in {fname} has no anchor"
+                    )
+                    continue
+                if "://" in href:
+                    continue
+                hrefs[href].add(fname)
+        for href, sources in hrefs.items():
+            assert os.path.exists(f"htmlcov/{href}"), (
+                f"These files link to {href!r}, which doesn't exist: {', '.join(sources)}"
+            )
+
 
 class FileWriteTracker:
     """A fake object to track how `open` is used to write files."""
@@ -291,6 +317,13 @@ class HtmlDeltaTest(HtmlTestHelpers, CoverageTest):
         with open("htmlcov/.gitignore") as fgi:
             assert fgi.read() == "# ignore nothing"
 
+    def test_dont_write_gitignore_into_existing_directory(self):
+        self.create_initial_files()
+        self.make_file("htmlcov/README", "My files: don't touch!")
+        self.run_coverage()
+        self.assert_doesnt_exist("htmlcov/.gitignore")
+        self.assert_exists("htmlcov/index.html")
+
 
 class HtmlTitleTest(HtmlTestHelpers, CoverageTest):
     """Tests of the HTML title support."""
@@ -356,7 +389,7 @@ class HtmlWithUnparsableFilesTest(HtmlTestHelpers, CoverageTest):
         assert_coverage_warnings(
             warns,
             re.compile(r"Couldn't parse Python file '.*innocuous.py' \(couldnt-parse\)"),
-            )
+        )
         self.assert_exists("htmlcov/index.html")
         # This would be better as a glob, if the HTML layout changes:
         self.assert_doesnt_exist("htmlcov/innocuous.html")
@@ -861,8 +894,8 @@ assert len(math) == 18
             gold_path("html/other"), "out/other",
             extra_scrubs=[
                 (r'href="d_[0-9a-z]{16}_', 'href="_TEST_TMPDIR_othersrc_'),
-                ],
-            )
+            ],
+        )
         contains(
             'out/other/index.html',
             '<a href="here_py.html">here.py</a>',
@@ -1089,7 +1122,7 @@ class HtmlWithContextsTest(HtmlTestHelpers, CoverageTest):
             assert b == (14-4)
             helper(
                 16
-                )
+            )
 
         test_one()
         x = 20
@@ -1157,3 +1190,21 @@ class HtmlWithContextsTest(HtmlTestHelpers, CoverageTest):
                 if label == ld.contexts_label or label in (ld.contexts or ())
             ]
             assert sorted(expected) == sorted(actual)
+
+
+class HtmlHelpersTest(HtmlTestHelpers, CoverageTest):
+    """Tests of the helpers in HtmlTestHelpers."""
+
+    def test_bad_link(self):
+        # Does assert_valid_hrefs detect links to non-existent files?
+        self.make_file("htmlcov/index.html", "<a href='nothing.html'>Nothing</a>")
+        msg = "These files link to 'nothing.html', which doesn't exist: htmlcov.index.html"
+        with pytest.raises(AssertionError, match=msg):
+            self.assert_valid_hrefs()
+
+    def test_bad_anchor(self):
+        # Does assert_valid_hrefs detect fragments that go nowhere?
+        self.make_file("htmlcov/index.html", "<a href='#nothing'>Nothing</a>")
+        msg = "Fragment '#nothing' in htmlcov.index.html has no anchor"
+        with pytest.raises(AssertionError, match=msg):
+            self.assert_valid_hrefs()
