@@ -4,6 +4,7 @@
 """Oddball cases for testing coverage.py"""
 
 import os.path
+import re
 import sys
 
 from flaky import flaky
@@ -15,6 +16,7 @@ from coverage.files import abs_file
 from coverage.misc import import_local_file
 
 from tests.coveragetest import CoverageTest
+from tests.helpers import swallow_warnings
 from tests import osinfo
 
 
@@ -81,7 +83,7 @@ class RecursionTest(CoverageTest):
     def test_long_recursion(self):
         # We can't finish a very deep recursion, but we don't crash.
         with pytest.raises(RuntimeError):
-            with pytest.warns(None):
+            with swallow_warnings("Trace function changed, data is likely wrong: None"):
                 self.check_coverage("""\
                     def recur(n):
                         if n == 0:
@@ -92,7 +94,7 @@ class RecursionTest(CoverageTest):
                     recur(100000)  # This is definitely too many frames.
                     """,
                     [1, 2, 3, 5, 7], ""
-                    )
+                )
 
     def test_long_recursion_recovery(self):
         # Test the core of bug 93: https://github.com/nedbat/coveragepy/issues/93
@@ -118,7 +120,7 @@ class RecursionTest(CoverageTest):
             """)
 
         cov = coverage.Coverage()
-        with pytest.warns(None):
+        with swallow_warnings("Trace function changed, data is likely wrong: None"):
             self.start_import_stop(cov, "recur")
 
         pytrace = (cov._collector.tracer_name() == "PyTracer")
@@ -132,9 +134,15 @@ class RecursionTest(CoverageTest):
 
         # Get a warning about the stackoverflow effect on the tracing function.
         if pytrace:                                 # pragma: no metacov
-            assert cov._warnings == ["Trace function changed, measurement is likely wrong: None"]
+            assert len(cov._warnings) == 1
+            assert re.fullmatch(
+                r"Trace function changed, data is likely wrong: None != " +
+                r"<bound method PyTracer._trace of " +
+                "<PyTracer at 0x[0-9a-fA-F]+: 5 lines in 1 files>>",
+                cov._warnings[0],
+            )
         else:
-            assert cov._warnings == []
+            assert not cov._warnings
 
 
 class MemoryLeakTest(CoverageTest):
@@ -338,25 +346,25 @@ class ExceptionTest(CoverageTest):
                 'doit.py': [302, 303, 304, 305],
                 'fly.py': [102, 103],
                 'oops.py': [2, 3],
-                }),
+            }),
             ("doit catch oops", {
                 'doit.py': [302, 303],
                 'catch.py': [202, 203, 204, 206, 207],
                 'oops.py': [2, 3],
-                }),
+            }),
             ("doit fly catch oops", {
                 'doit.py': [302, 303],
                 'fly.py': [102, 103, 104],
                 'catch.py': [202, 203, 204, 206, 207],
                 'oops.py': [2, 3],
-                }),
+            }),
             ("doit catch fly oops", {
                 'doit.py': [302, 303],
                 'catch.py': [202, 203, 204, 206, 207],
                 'fly.py': [102, 103],
                 'oops.py': [2, 3],
-                }),
-            ]
+            }),
+        ]
 
         for callnames, lines_expected in runs:
 
