@@ -14,7 +14,7 @@ from coverage.exceptions import NotPython
 from coverage.parser import ast_dump, ast_parse, PythonParser
 
 from tests.coveragetest import CoverageTest, TESTS_DIR
-from tests.helpers import arcz_to_arcs, re_lines
+from tests.helpers import arcz_to_arcs, re_lines, xfail_pypy_3749
 
 
 class PythonParserTest(CoverageTest):
@@ -43,8 +43,8 @@ class PythonParserTest(CoverageTest):
                 pass
             """)
         assert parser.exit_counts() == {
-            2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1
-            }
+            2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1
+        }
 
     def test_generator_exit_counts(self):
         # https://github.com/nedbat/coveragepy/issues/324
@@ -76,7 +76,7 @@ class PythonParserTest(CoverageTest):
             """)
         assert parser.exit_counts() == {
             1: 1, 2:1, 3:2, 4:1, 5:2, 6:1, 7:1, 8:1, 9:1
-            }
+        }
 
     def test_excluded_classes(self):
         parser = self.parse_source("""\
@@ -89,8 +89,8 @@ class PythonParserTest(CoverageTest):
                     pass
             """)
         assert parser.exit_counts() == {
-            1:1, 2:1, 3:1
-            }
+            1:0, 2:1, 3:1
+        }
 
     def test_missing_branch_to_excluded_code(self):
         parser = self.parse_source("""\
@@ -139,10 +139,7 @@ class PythonParserTest(CoverageTest):
                 '''
                 """)
 
-    @pytest.mark.xfail(
-        env.PYPY and env.PYPYVERSION == (7, 3, 0),
-        reason="https://bitbucket.org/pypy/pypy/issues/3139",
-    )
+    @xfail_pypy_3749
     def test_decorator_pragmas(self):
         parser = self.parse_source("""\
             # 1
@@ -178,6 +175,7 @@ class PythonParserTest(CoverageTest):
         assert parser.raw_statements == raw_statements
         assert parser.statements == {8}
 
+    @xfail_pypy_3749
     def test_decorator_pragmas_with_colons(self):
         # A colon in a decorator expression would confuse the parser,
         # ending the exclusion of the decorated function.
@@ -212,6 +210,7 @@ class PythonParserTest(CoverageTest):
         assert parser.raw_statements == {1, 2, 3, 5, 6, 7, 8}
         assert parser.statements == {1, 2, 3}
 
+    @xfail_pypy_3749
     def test_empty_decorated_function(self):
         parser = self.parse_source("""\
             def decorator(func):
@@ -240,10 +239,23 @@ class PythonParserTest(CoverageTest):
             expected_arcs.update(set(arcz_to_arcs("-46 6-4")))
             expected_exits.update({6: 1})
 
+        if env.PYBEHAVIOR.trace_decorator_line_again:
+            expected_arcs.update(set(arcz_to_arcs("54 98")))
+            expected_exits.update({9: 2, 5: 2})
+
         assert expected_statements == parser.statements
         assert expected_arcs == parser.arcs()
         assert expected_exits == parser.exit_counts()
 
+    def test_fuzzed_double_parse(self):
+        # https://bugs.chromium.org/p/oss-fuzz/issues/detail?id=50381
+        # The second parse used to raise `TypeError: 'NoneType' object is not iterable`
+        msg = "EOF in multi-line statement"
+        with pytest.raises(NotPython, match=msg):
+            self.parse_source("]")
+        with pytest.raises(NotPython, match=msg):
+            self.parse_source("]")
+
 
 class ParserMissingArcDescriptionTest(CoverageTest):
     """Tests for PythonParser.missing_arc_description."""
@@ -429,10 +441,10 @@ class ParserMissingArcDescriptionTest(CoverageTest):
             """)
         assert parser.missing_arc_description(3, 4) == (
             "line 3 didn't jump to line 4, because the pattern on line 3 never matched"
-            )
+        )
         assert parser.missing_arc_description(3, 5) == (
             "line 3 didn't jump to line 5, because the pattern on line 3 always matched"
-            )
+        )
 
 
 class ParserFileTest(CoverageTest):
@@ -460,7 +472,7 @@ class ParserFileTest(CoverageTest):
             class Bar:
                 pass
             """
-        counts = { 2:2, 3:1, 4:2, 5:1, 7:1, 9:2, 10:1 }
+        counts = { 2:1, 3:1, 4:2, 5:1, 7:1, 9:1, 10:1 }
         fname = slug + ".py"
         self.make_file(fname, text, newline=newline)
         parser = self.parse_file(fname)
@@ -506,8 +518,9 @@ def test_ast_dump():
     # Run the AST_DUMP code to make sure it doesn't fail, with some light
     # assertions. Use parser.py as the test code since it is the longest file,
     # and fitting, since it's the AST_DUMP code.
+    import coverage.parser
     files = [
-        os.path.join(TESTS_DIR, "../coverage/parser.py"),
+        coverage.parser.__file__,
         os.path.join(TESTS_DIR, "stress_phystoken.tok"),
     ]
     for fname in files:
