@@ -173,8 +173,6 @@ class SourceEncodingTest(CoverageTest):
         for _, source, expected in ENCODING_DECLARATION_SOURCES:
             assert source_encoding(source) == expected, f"Wrong encoding in {source!r}"
 
-    # PyPy3 gets this case wrong. Not sure what I can do about it, so skip the test.
-    @pytest.mark.skipif(env.PYPY, reason="PyPy3 is wrong about non-comment encoding. Skip it.")
     def test_detect_source_encoding_not_in_comment(self):
         # Should not detect anything here
         source = b'def parse(src, encoding=None):\n    pass'
