@@ -5,6 +5,7 @@
 
 import inspect
 import io
+import math
 import os.path
 from xml.etree import ElementTree
 
@@ -20,7 +21,7 @@ from coverage.misc import import_local_file
 import coverage.plugin
 
 from tests.coveragetest import CoverageTest
-from tests.helpers import CheckUniqueFilenames
+from tests.helpers import CheckUniqueFilenames, swallow_warnings
 
 
 class FakeConfig:
@@ -193,8 +194,9 @@ class PluginTest(CoverageTest):
         cov = coverage.Coverage(debug=["sys"])
         cov._debug_file = debug_out
         cov.set_option("run:plugins", ["plugin_sys_info"])
-        with pytest.warns(None):
-            # Catch warnings so we don't see "plugins aren't supported on PyTracer"
+        with swallow_warnings(
+            r"Plugin file tracers \(plugin_sys_info.Plugin\) aren't supported with PyTracer"
+        ):
             cov.start()
         cov.stop()      # pragma: nested
 
@@ -208,7 +210,7 @@ class PluginTest(CoverageTest):
             "-- sys: plugin_sys_info.Plugin -------------------------------",
             "hello: world",
             "-- end -------------------------------------------------------",
-            ]
+        ]
         assert expected_end == out_lines[-len(expected_end):]
 
     def test_plugin_with_no_sys_info(self):
@@ -234,7 +236,7 @@ class PluginTest(CoverageTest):
         expected_end = [
             "-- sys: plugin_no_sys_info.Plugin ----------------------------",
             "-- end -------------------------------------------------------",
-            ]
+        ]
         assert expected_end == out_lines[-len(expected_end):]
 
     def test_local_files_are_importable(self):
@@ -424,9 +426,9 @@ class GoodFileTracerTest(FileTracerTest):
             'foo_7.html       7      5      0      0    29%   1-3, 6-7',
             '--------------------------------------------------------',
             'TOTAL           11      7      0      0    36%',
-            ]
+        ]
         assert expected == report
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
     def test_plugin2_with_html_report(self):
         self.make_render_and_caller()
@@ -437,7 +439,7 @@ class GoodFileTracerTest(FileTracerTest):
         self.start_import_stop(cov, "caller")
 
         total = cov.html_report(include=["*.html"], omit=["uni*.html"])
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
         self.assert_exists("htmlcov/index.html")
         self.assert_exists("htmlcov/bar_4_html.html")
@@ -452,7 +454,7 @@ class GoodFileTracerTest(FileTracerTest):
         self.start_import_stop(cov, "caller")
 
         total = cov.xml_report(include=["*.html"], omit=["uni*.html"])
-        assert round(abs(total-36.36), 2) == 0
+        assert math.isclose(total, 4 / 11 * 100)
 
         dom = ElementTree.parse("coverage.xml")
         classes = {}
@@ -523,7 +525,7 @@ class GoodFileTracerTest(FileTracerTest):
             'unsuspecting.py       6      3    50%   2, 4, 6',
             '-----------------------------------------------',
             'TOTAL                 6      3    50%',
-            ]
+        ]
         assert expected == report
         assert total == 50
 
