@@ -302,7 +302,7 @@ class ProcessTest(CoverageTest):
         status, out = self.run_command_status("coverage run throw.py")
         out2 = self.run_command("python throw.py")
         if env.PYPY:
-            # Pypy has an extra frame in the traceback for some reason
+            # PyPy has an extra frame in the traceback for some reason
             out2 = re_lines_text("toplevel", out2, match=False)
         assert out == out2
 
@@ -561,9 +561,7 @@ class ProcessTest(CoverageTest):
             print("FOOEY == %s" % os.getenv("FOOEY"))
             """)
 
-        fullcov = os.path.join(
-            os.path.dirname(coverage.__file__), "fullcoverage"
-            )
+        fullcov = os.path.join(os.path.dirname(coverage.__file__), "fullcoverage")
         self.set_environ("FOOEY", "BOO")
         self.set_environ("PYTHONPATH", fullcov)
         out = self.run_command("python -X frozen_modules=off -m coverage run -L getenv.py")
@@ -703,14 +701,6 @@ class EnvironmentTest(CoverageTest):
 
         expected = self.run_command("python with_main")
         actual = self.run_command("coverage run with_main")
-
-        # PyPy includes the current directory in the path when running a
-        # directory, while CPython and coverage.py do not.  Exclude that from
-        # the comparison also...
-        if env.PYPY:
-            ignored = re.escape(os.getcwd())
-            expected = re_lines_text(ignored, expected, match=False)
-            actual = re_lines_text(ignored, actual, match=False)
         self.assert_tryexecfile_output(expected, actual)
 
     def test_coverage_run_dashm_dir_no_init_is_like_python(self):
@@ -828,7 +818,7 @@ class EnvironmentTest(CoverageTest):
             sys.path[0:0] = [
                 r'{abc}',
                 '/Users/somebody/temp/something/eggs/something-4.5.1-py2.7-xxx-10.13-x86_64.egg',
-                ]
+            ]
 
             import coverage.cmdline
 
@@ -983,34 +973,33 @@ class AliasedCommandTest(CoverageTest):
     run_in_temp_dir = False
 
     def test_major_version_works(self):
-        # "coverage2" works on py2
+        # "coverage3" works on py3
         cmd = "coverage%d" % sys.version_info[0]
         out = self.run_command(cmd)
         assert "Code coverage for Python" in out
 
     def test_wrong_alias_doesnt_work(self):
-        # "coverage3" doesn't work on py2
+        # "coverage2" doesn't work on py3
         assert sys.version_info[0] in [2, 3]    # Let us know when Python 4 is out...
         badcmd = "coverage%d" % (5 - sys.version_info[0])
         out = self.run_command(badcmd)
         assert "Code coverage for Python" not in out
 
     def test_specific_alias_works(self):
-        # "coverage-2.7" works on py2.7
+        # "coverage-3.9" works on py3.9
         cmd = "coverage-%d.%d" % sys.version_info[:2]
         out = self.run_command(cmd)
         assert "Code coverage for Python" in out
 
-    def test_aliases_used_in_messages(self):
-        cmds = [
-            "coverage",
-            "coverage%d" % sys.version_info[0],
-            "coverage-%d.%d" % sys.version_info[:2],
-        ]
-        for cmd in cmds:
-            out = self.run_command(f"{cmd} foobar")
-            assert "Unknown command: 'foobar'" in out
-            assert f"Use '{cmd} help' for help" in out
+    @pytest.mark.parametrize("cmd", [
+        "coverage",
+        "coverage%d" % sys.version_info[0],
+        "coverage-%d.%d" % sys.version_info[:2],
+    ])
+    def test_aliases_used_in_messages(self, cmd):
+        out = self.run_command(f"{cmd} foobar")
+        assert "Unknown command: 'foobar'" in out
+        assert f"Use '{cmd} help' for help" in out
 
 
 class PydocTest(CoverageTest):
@@ -1079,7 +1068,7 @@ class FailUnderTest(CoverageTest):
             "b = 2\n" * 2000 +
             "if a > 3:\n" +
             "    c = 4\n"
-            )
+        )
         self.make_data_file(lines={abs_file("ninety_nine_plus.py"): range(1, 2002)})
         st, out = self.run_command_status("coverage report --fail-under=100")
         assert st == 2
